# Todo: these setting are not fully implemented through out all of the code

DJANGO_GLUE_CONTEXT_NAME = 'django_glue'
DJANGO_GLUE_SESSION_NAME = 'django_glue'

DJANGO_GLUE_KEEP_LIVE_CONTEXT_NAME = 'django_glue_keep_live'
DJANGO_GLUE_KEEP_LIVE_SESSION_NAME = 'django_glue_keep_live'

DJANGO_GLUE_KEEP_LIVE_EXPIRE_TIME_SECONDS = 120.0
DJANGO_GLUE_KEEP_LIVE_INTERVAL_TIME_SECONDS = 90.0

DJANGO_GLUE_AUTO_DELETE = False

DJANGO_GLUE_URL = 'django_glue/'
